
// LCD module connections
sbit LCD_RS at RC0_bit;
sbit LCD_EN at RC1_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISC0_bit;
sbit LCD_EN_Direction at TRISC1_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

unsigned char key;
unsigned char flag;
int counter = 0;
const char Less_Than[] = {1,3,7,15,15,7,3,1};
const char Greater_Than[] = {16,24,28,30,30,28,24,16};
const char Shaded_Block[] = {21,10,21,10,21,10,21,10};

void interrupt(void)
{
 if(INTCON.RBIE & INTCON.RBIF)
 {
  key = PORTB;
  key = key & 0xC0;
  if(key == 0x40)             //Down key is Pressed
  {
   if(counter > -16)
   {
    counter--;
   }
  }
  else if(key == 0x80)        //Up Key Pressed
  {
   if(counter < 16)
   {
    counter++;
   }
  }
  key = PORTB;
  key = key & 0xC0;
  while(key != 0xC0)
  {
   PORTB = 0xC0;
   key = PORTB;
   key = key & 0xC0;
  }
  flag = 1;
  INTCON.RBIF = 0;
 }
}

void Display_Value(int value)
{
 unsigned char ones,tens;
 if(value >= 0)
 {
  Lcd_Chr(1,14,'+');
 }
 else
 {
  value = value * (-1);
  Lcd_Chr(1,14,'-');
 }
 ones = value % 10;
 value = value / 10;
 tens = value % 10;
 ones = ones | 0x30;
 tens = tens | 0x30;
 Lcd_Chr(1,15,tens);
 Lcd_Chr(1,16,ones);
}

void Progress_Bar(int value)
{
 unsigned char j;
 if(value == 0)
 {
  Lcd_Chr(2,8,0);
  Lcd_Chr(2,9,1);
 }
 else if(value > 0)
 {
  value = value+16;
  Lcd_Chr(2,8,' ');
  for(j=16;j<value;j++)
  {
   if(j%2 == 0)
   {
    Lcd_Chr(2,j/2+1,2);
   }
   else
   {
    Lcd_Chr(2,j/2+1,0xFF);
   }
  }
  j=j+1;
  for(;j<32;j++)
  {
   if(j%2 == 0)
   {
    Lcd_Chr(2,j/2+1,' ');
   }
   else
   {
    Lcd_Chr(2,j/2+1,' ');
   }
  }
 }
 else if(value < 0)
 {
  value = value + 16;
  Lcd_Chr(2,9,' ');
 }
}

void main()
{
 unsigned char i;
 
 counter = 0;
 TRISB |= 0b11110000;
 Lcd_Init();
 Lcd_Cmd(_LCD_CLEAR);
 Lcd_Cmd(_LCD_CURSOR_OFF);
 Delay_ms(10);
 Lcd_Out(1,5,"EDA BOARD");
 Lcd_Out(2,1,"<-PROGRESS-BAR->");
 Delay_ms(1000);
 
 INTCON.RBIE = 1;
 INTCON.GIE = 1;
 INTCON.PEIE = 1;
 
 Lcd_Cmd(_LCD_CLEAR);
 Lcd_Out(1,1,"Progress Bar:");
 Display_Value(counter);
 Progress_Bar(counter);
 /********************************************/
 Lcd_Cmd(64);          //CGRAM Address
 for(i=0;i<8;i++)
 {
  Lcd_Chr_Cp(Less_Than[i]);
 }
 Lcd_Cmd(_LCD_RETURN_HOME);
 
 Lcd_Cmd(72);
 for(i=0;i<8;i++)
 {
  Lcd_Chr_Cp(Greater_Than[i]);
 }
 Lcd_Cmd(_LCD_RETURN_HOME);
 
 Lcd_Cmd(80);
 for(i=0;i<8;i++)
 {
  Lcd_Chr_Cp(Shaded_Block[i]);
 }
 Lcd_Cmd(_LCD_RETURN_HOME);
 /********************************************/
 while(1)
 {
  if(flag != 0)
  {
   Lcd_Out(1,1,"Progress Bar:");
   Display_Value(counter);
   Progress_Bar(counter);
   flag = 0;
  }
 }
}